@@ -50,5 +50,7 @@ void HandleKeyPress(void);
 void HandleVisibilityNotify(void);
 STROKE_CODE(void HandleButtonRelease(void);)
 STROKE_CODE(void HandleMotionNotify(void);)
+void WaitForButtonsUp(Bool do_handle_expose);
+int discard_events(long event_mask);
 
 #endif /* _EVENTS_ */
